@@ -267,11 +267,6 @@ KConfigPropertyMap *Applet::configuration()
     return d->configPropertyMap;
 }
 
-KPackage::Package Applet::kPackage() const
-{
-    return d->package;
-}
-
 void Applet::updateConstraints(Constraints constraints)
 {
     d->scheduleConstraintsUpdate(constraints);
@@ -609,17 +604,6 @@ void Applet::flushPendingConstraintsEvents()
     // pass the constraint on to the actual subclass
     constraintsEvent(c);
 
-    if (c & StartupCompletedConstraint) {
-        // start up is done, we can now go do a mod timer
-        if (d->modificationsTimer) {
-            if (d->modificationsTimer->isActive()) {
-                d->modificationsTimer->stop();
-            }
-        } else {
-            d->modificationsTimer = new QBasicTimer;
-        }
-    }
-
     if (c & FormFactorConstraint) {
         Q_EMIT formFactorChanged(formFactor());
     }
@@ -813,21 +797,40 @@ void Applet::configChanged()
     }
 }
 
-QString Applet::filePath(const QByteArray &key, const QString &filename) const
+QUrl Applet::fileUrl(const QByteArray &key, const QString &filename) const
 {
     if (d->package.isValid()) {
-        return d->package.filePath(key, filename);
+        return d->package.fileUrl(key, filename);
     }
-    return QString();
+    return QUrl();
+}
+
+QUrl Applet::mainScript() const
+{
+    if (d->package.isValid()) {
+        return d->package.fileUrl("mainscript");
+    }
+    return QUrl();
+}
+
+QUrl Applet::configModel() const
+{
+    if (d->package.isValid()) {
+        return d->package.fileUrl("configmodel");
+    }
+
+    return QUrl();
+}
+
+bool Applet::sourceValid() const
+{
+    return d->package.isValid();
 }
 
 void Applet::timerEvent(QTimerEvent *event)
 {
     if (d->transient) {
         d->constraintsTimer.stop();
-        if (d->modificationsTimer) {
-            d->modificationsTimer->stop();
-        }
         return;
     }
 
@@ -839,13 +842,6 @@ void Applet::timerEvent(QTimerEvent *event)
         if (!(d->pendingConstraints & StartupCompletedConstraint)) {
             flushPendingConstraintsEvents();
         }
-    } else if (d->modificationsTimer && event->timerId() == d->modificationsTimer->timerId()) {
-        d->modificationsTimer->stop();
-        // invalid group, will result in save using the default group
-        KConfigGroup cg;
-
-        save(cg);
-        Q_EMIT configNeedsSaving();
     }
 }
 
@@ -861,6 +857,16 @@ bool Applet::isContainment() const
     return qobject_cast<const Containment *>(this) && qobject_cast<Corona *>(parent());
 }
 
+QString Applet::translationDomain() const
+{
+    const QString rootPath = d->appletDescription.value(QStringLiteral("X-Plasma-RootPath"));
+    if (!rootPath.isEmpty()) {
+        return QLatin1String("plasma_applet_") + rootPath;
+    } else {
+        return QLatin1String("plasma_applet_") + d->appletDescription.pluginId();
+    }
+}
+
 } // Plasma namespace
 
 #include "moc_applet.cpp"
