@@ -618,6 +618,13 @@ public:
      **/
     bool hasConfigurationInterface() const;
 
+    /**
+     * The translation domain for this applet
+     *
+     * @since 6.1
+     */
+    QString translationDomain() const;
+
 Q_SIGNALS:
     // BOOKKEEPING
     /**
@@ -868,8 +875,10 @@ protected:
     void timerEvent(QTimerEvent *event) override;
 
 private:
-    QString filePath(const QByteArray &key, const QString &filename = QString()) const;
-    KPackage::Package kPackage() const;
+    QUrl fileUrl(const QByteArray &key, const QString &filename) const;
+    QUrl mainScript() const;
+    QUrl configModel() const;
+    bool sourceValid() const;
     /**
      * @internal This constructor is to be used with the Package loading system.
      *
