@@ -80,7 +80,7 @@ void Containment::init()
         }
     });
     connect(corona(), &Plasma::Corona::screenGeometryChanged, this, [this](int screenId) {
-        if (screenId == screen() || screenId == lastScreen()) {
+        if (screenId == screen()) {
             Q_EMIT screenGeometryChanged(screenGeometry());
         }
     });
@@ -106,15 +106,6 @@ void Containment::init()
             configAction->setText(i18nc("%1 is the name of the applet", "Configure %1...", title()));
         }
     }
-    connect(corona(), &Plasma::Corona::editModeChanged, this, [=, this](bool isEditModeNow) {
-        if (d->type == Containment::Type::Panel || d->type == Containment::Type::CustomPanel) {
-            if (isEditModeNow) {
-                configAction->setText(i18n("Exit Edit Mode"));
-            } else {
-                configAction->setText(i18n("Enter Edit Mode"));
-            }
-        }
-    });
 
     QAction *appletBrowserAction = internalAction(QStringLiteral("add widgets"));
     if (appletBrowserAction) {
@@ -450,10 +441,6 @@ void Containment::addApplet(Applet *applet, const QRectF &geometryHint)
 
         applet->init();
 
-        if (isNew) {
-            applet->save(*applet->d->mainConfigGroup());
-            Q_EMIT configNeedsSaving();
-        }
         // FIXME: an on-appear animation would be nice to have again
     }
 
@@ -469,7 +456,8 @@ void Containment::addApplet(Applet *applet, const QRectF &geometryHint)
         applet->flushPendingConstraintsEvents();
     }
 
-    applet->d->scheduleModificationNotification();
+    applet->save(*applet->d->mainConfigGroup());
+    Q_EMIT configNeedsSaving();
 }
 
 QList<Applet *> Containment::applets() const
@@ -587,6 +575,14 @@ void Containment::setWallpaperGraphicsObject(QObject *object)
     Q_EMIT wallpaperGraphicsObjectChanged();
 }
 
+QUrl Containment::compactApplet() const
+{
+    if (Applet::d->package.isValid()) {
+        return Applet::d->package.fileUrl("compactapplet");
+    }
+    return QUrl();
+}
+
 void Containment::setContainmentActions(const QString &trigger, const QString &pluginName)
 {
     KConfigGroup cfg = d->containmentActionsConfig();
