@@ -25,14 +25,12 @@
 #include <KConfigLoader>
 #include <KGlobalAccel>
 #include <KLocalizedString>
-#include <kpackage/packageloader.h>
 
 #include "containment.h"
 #include "corona.h"
 #include "debug_p.h"
 #include "pluginloader.h"
 #include "private/containment_p.h"
-#include "timetracker.h"
 
 namespace Plasma
 {
@@ -50,7 +48,6 @@ AppletPrivate::AppletPrivate(const KPluginMetaData &info, int uniqueID, Applet *
     , actions(AppletPrivate::defaultActions(applet))
     , activationAction(nullptr)
     , itemStatus(Types::UnknownStatus)
-    , modificationsTimer(nullptr)
     , deleteNotificationTimer(nullptr)
     , hasConfigurationInterface(false)
     , failed(false)
@@ -67,11 +64,6 @@ AppletPrivate::AppletPrivate(const KPluginMetaData &info, int uniqueID, Applet *
         s_maxAppletId = appletId;
     }
     QObject::connect(actions.value(QStringLiteral("configure")), SIGNAL(triggered()), q, SLOT(requestConfiguration()));
-#ifndef NDEBUG
-    if (qEnvironmentVariableIsSet("PLASMA_TRACK_STARTUP")) {
-        new TimeTracker(q);
-    }
-#endif
 
     for (auto it = actions.constBegin(); it != actions.constEnd(); ++it) {
         QAction *action = it.value();
@@ -92,7 +84,6 @@ AppletPrivate::~AppletPrivate()
     configLoader = nullptr;
     delete mainConfig;
     mainConfig = nullptr;
-    delete modificationsTimer;
 }
 
 void AppletPrivate::init(const QVariantList &args)
@@ -122,26 +113,6 @@ void AppletPrivate::init(const QVariantList &args)
         return;
     }
 
-    // A constructor may have set a valid package already
-    if (!package.isValid()) {
-        const QString packagePath = QFileInfo(appletDescription.fileName()).dir().path();
-        QString path = appletDescription.value(QStringLiteral("X-Plasma-RootPath"));
-        if (path.isEmpty()) {
-            path = packagePath.isEmpty() ? appletDescription.pluginId() : packagePath;
-        }
-
-        package = KPackage::PackageLoader::self()->loadPackage(QStringLiteral("Plasma/Applet"));
-        package.setPath(path);
-
-        if (!package.isValid()) {
-            q->setLaunchErrorMessage(i18nc("Package file, name of the widget",
-                                           "Could not open the %1 package required for the %2 widget.",
-                                           appletDescription.pluginId(),
-                                           appletDescription.name()));
-            return;
-        }
-    }
-
     if (!q->isContainment()) {
         QAction *a = new QAction(QIcon::fromTheme(QStringLiteral("widget-alternatives")), i18n("Show Alternatives..."), q);
         a->setVisible(false);
@@ -157,7 +128,14 @@ void AppletPrivate::init(const QVariantList &args)
 
             const QStringList provides = q->pluginMetaData().value(QStringLiteral("X-Plasma-Provides"), QStringList());
             if (!provides.isEmpty() && q->immutability() == Types::Mutable) {
-                auto filter = [&provides](const KPluginMetaData &md) -> bool {
+                const auto applets = Plasma::PluginLoader::self()->listAppletMetaData(QString());
+
+                auto filter = [this, &provides](const KPluginMetaData &md) -> bool {
+                    // Ignore ourselves
+                    if (q->pluginMetaData().pluginId() == md.pluginId()) {
+                        return false;
+                    }
+
                     const QStringList provided = md.value(QStringLiteral("X-Plasma-Provides"), QStringList());
                     for (const QString &p : provides) {
                         if (provided.contains(p)) {
@@ -166,11 +144,8 @@ void AppletPrivate::init(const QVariantList &args)
                     }
                     return false;
                 };
-                QList<KPluginMetaData> applets = KPackage::PackageLoader::self()->findPackages(QStringLiteral("Plasma/Applet"), QString(), filter);
 
-                if (applets.count() > 1) {
-                    hasAlternatives = true;
-                }
+                hasAlternatives = std::any_of(applets.cbegin(), applets.cend(), filter);
             }
             a->setVisible(hasAlternatives);
         });
@@ -342,7 +317,7 @@ void AppletPrivate::globalShortcutChanged()
     QString oldShortCut = shortcutConfig.readEntry("global", QString());
     if (newShortCut != oldShortCut) {
         shortcutConfig.writeEntry("global", newShortCut);
-        scheduleModificationNotification();
+        Q_EMIT q->configNeedsSaving();
     }
     // qCDebug(LOG_PLASMA) << "after" << shortcut.primary() << d->activationAction->globalShortcut().primary();
 }
@@ -458,12 +433,6 @@ void AppletPrivate::setupPackage()
     //         << ", main script is" << package->filePath("mainscript");
 #endif
 
-    // FIXME: Replace with ki18n functionality once semantics is clear.
-    // const QString translationsPath = package->filePath("translations");
-    // if (!translationsPath.isEmpty()) {
-    //     KGlobal::dirs()->addResourceDir("locale", translationsPath);
-    // }
-
     if (!package.filePath("mainconfigui").isEmpty()) {
         q->setHasConfigurationInterface(true);
     }
@@ -496,15 +465,6 @@ void AppletPrivate::scheduleConstraintsUpdate(Applet::Constraints c)
     pendingConstraints |= c;
 }
 
-void AppletPrivate::scheduleModificationNotification()
-{
-    // modificationsTimer is not allocated until we get our notice of being started
-    if (modificationsTimer) {
-        // schedule a save
-        modificationsTimer->start(1000, q);
-    }
-}
-
 KConfigGroup *AppletPrivate::mainConfigGroup()
 {
     if (mainConfig) {
