@@ -48,6 +48,7 @@ public:
     void updateMinimumHeight();
     void updateMaximumWidth();
     void updateMaximumHeight();
+    void updateTitle();
     void mainItemLoaded();
 
     ConfigView *q;
@@ -91,17 +92,12 @@ void ConfigViewPrivate::init()
     applet.data()->setUserConfiguring(true);
 
     KLocalizedContext *localizedContextObject = new KLocalizedContext(q->engine());
-    const QString rootPath = applet.data()->pluginMetaData().value(QStringLiteral("X-Plasma-RootPath"));
-    if (!rootPath.isEmpty()) {
-        localizedContextObject->setTranslationDomain(QStringLiteral("plasma_applet_") + rootPath);
-    } else {
-        localizedContextObject->setTranslationDomain(QStringLiteral("plasma_applet_") + applet.data()->pluginMetaData().pluginId());
-    }
+    localizedContextObject->setTranslationDomain(applet->translationDomain());
     rootContext->setContextObject(localizedContextObject);
 
     // FIXME: problem on nvidia, all windows should be transparent or won't show
     q->setColor(Qt::transparent);
-    q->setTitle(i18n("%1 Settings", applet.data()->title()));
+    updateTitle();
 
     // systray case
     if (!applet.data()->containment()->corona()) {
@@ -131,7 +127,7 @@ void ConfigViewPrivate::init()
     }
 
     // config model local of the applet
-    QQmlComponent component(q->engine(), applet.data()->kPackage().fileUrl("configmodel"));
+    QQmlComponent component(q->engine(), applet.data()->configModel());
     QObject *object = component.create(rootContext);
     configModel = qobject_cast<ConfigModel *>(object);
 
@@ -229,12 +225,24 @@ void ConfigViewPrivate::updateMaximumHeight()
     }
 }
 
+void ConfigViewPrivate::updateTitle()
+{
+    QVariant itemTitle = rootItem ? rootItem->property("title") : QVariant();
+    q->setTitle(itemTitle.canConvert<QString>() ? i18n("%1 — %2 Settings", itemTitle.toString(), applet.data()->title())
+                                                : i18n("%1 Settings", applet.data()->title()));
+}
+
 void ConfigViewPrivate::mainItemLoaded()
 {
     if (applet) {
         KConfigGroup cg = applet.data()->config();
         cg = KConfigGroup(&cg, QStringLiteral("ConfigDialog"));
         q->resize(cg.readEntry("DialogWidth", q->width()), cg.readEntry("DialogHeight", q->height()));
+
+        if (rootItem->property("title").isValid()) {
+            QObject::connect(rootItem, SIGNAL(titleChanged()), q, SLOT(updateTitle()));
+            updateTitle();
+        }
     }
 
     // Extract the representation's Layout, if any
